<?php declare(strict_types=1);

namespace uib\ub\loadspeakr;


final class Endpoint
{
    private string $sparqlUrl;
    private array $params;

    public function __construct(string $sparqlUrl, array $params)
    {
        $this->sparqlUrl = $sparqlUrl;
        $this->params = $params;
    }

    public function query($q, $output = 'json')
    {
        global $conf;

        if (!empty($this->params['output'])) {
            $auxoutput = $this->params['output'];
        }

        $accept = 'application/sparql-results+json';
        $this->params['output'] = $output;

        if ($output === 'xml') {
            $accept = 'application/sparql-results+xml';
        } elseif ($output === 'rdf') {
            $accept = 'application/rdf+xml';
        }

        $modified = 0;
        $now = time();
        $cacheFile = "";

        if (!empty($conf['cache']['global']) && is_int($conf['cache']['global']) && $conf['cache']['global'] > 0) {
            $cacheFile = $conf['home'] . "cache/query" . md5($this->sparqlUrl . $q);
            if (file_exists($cacheFile)) {
                $modified = filemtime($cacheFile);
            }
        }

        /**
         * If we have cached content use it if not make a https request.
         */
        if (is_int(
            $conf['cache']['global']
          ) && $conf['cache']['global'] > 0 && $modified + $conf['cache']['global'] > $now) {

            if ($conf['debug']) {
                $msg = "Taking data from cache ($cacheFile). Renewal in " . ($modified + $conf['cache']['global'] - $now) . " seconds\n";
                Logging::log($msg);
                echo $msg;
            }

            $aux = (file_get_contents($cacheFile));
        } else {

            $result = $this->curl($q, $accept, $conf);
            $aux = $result['result'];
            $http_status = $result['status'];

            if (is_int($conf['cache']['global']) && $conf['cache']['global'] > 0 && $http_status == 200) {
                file_put_contents($cacheFile, ($aux), LOCK_EX);
                if ($conf['debug']) {
                    $msg = "Notice: Writing results in " . $cacheFile . "\n";
                    Logging::log($msg, E_USER_NOTICE);
                    echo($msg);
                }
            }
        }

        if (false !== stripos($q, "select")) {
            $r = json_decode($aux, true);

            if ($conf['debug']) {
                if ($r == false) {
                    Logging::log("Warning: Results from a SELECT sparql query couldn't get parsed", E_USER_WARNING);
                    echo("Warning: Results from a SELECT sparql query couldn't get parsed");
                }
            }

            return $r;
        }

        if (false !== stripos($q, "describe")) {
            return $aux;
        }

        if (false !== stripos($q, "construct")) {
            return $aux;
        }

        if (false !== stripos($q, "ask")) {
            $r = json_decode($aux, true);
            return $r;
        }
    }

    private function curl(string $request, $accept, array $conf): array
    {
        $resource = curl_init();
        $context = ['Connection: close', 'Accept: ' . $accept];
        $params = $this->params;
        $params['query'] = $request;
        $url = $this->sparqlUrl . '?' . http_build_query($params, '', '&');
        curl_setopt($resource, CURLOPT_URL, $url);
        curl_setopt($resource, CURLOPT_HTTPHEADER, $context);
        curl_setopt($resource, CURLOPT_USERAGENT, "LODSPeaKr version " . $conf['version']);
        curl_setopt($resource, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($resource);
        $http_status = (int)curl_getinfo($resource, CURLINFO_HTTP_CODE);
        curl_close($resource);

        return [
            'result' => $result,
            'status' => $http_status,
        ];
    }

    public function queryPost($q)
    {
        $params = $this->params;
        $params['query'] = $q;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->sparqlUrl);
        curl_setopt($ch, CURLOPT_POST, count($params));
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        //execute post
        $result = curl_exec($ch);
        return $result;
    }

    public function getSparqlURL(): string
    {
        return $this->sparqlUrl;
    }

}
